from typing import Dict
from .base import BaseModelService
from .deepseek import DeepseekModelService
from .glm import GLMModelService


def create_model_service(provider: str, url: str, api_key: str) -> BaseModelService:
    """
    Create and return an appropriate model service instance.

    Parameters:
    - provider: Name of the service provider
    - url: Service URL
    - api_key: API key

    Returns:
    - An instance of a BaseModelService subclass
    """
    service_map: Dict[str, type] = {
        "zhipu": GLMModelService,
        "deepseek": DeepseekModelService,
    }

    service_class = service_map.get(provider.lower())
    if not service_class:
        raise ValueError(f"Unsupported service provider: {provider}")

    return service_class(provider, url, api_key)


__all__ = ["create_model_service"]



import datetime
import json
from zhipuai import ZhipuAI
from .base import BaseModelService
from typing import AsyncGenerator
from ..define import ChatRequest


class GLMModelService(BaseModelService):
    def __init__(self, provider: str, url: str, api_key: str):
        super().__init__(provider, url, api_key)
        self.client = ZhipuAI(api_key=self.api_key)

    async def chat(
        self,
        chat_request: ChatRequest,
    ) -> AsyncGenerator[str, None]:
        """
        Implement streaming chat functionality for GLM models using the Zhipu SDK.

        Parameters:
        - chat_request: Chat request object

        Returns:
        - Asynchronous generator producing strings formatted as SSE
        """
        messages_json = [message.model_dump() for message in chat_request.messages]

        model_name = chat_request.model.replace(":", "-", 1)

        # Use Zhipu SDK to make the request
        response = self.client.chat.completions.create(
            model=model_name, messages=messages_json, stream=True
        )

        # Parse the response
        for chunk in response:
            # check if finish_reason is a property of chunk
            finish_reason = chunk.choices[0].finish_reason
            end = True if finish_reason == "stop" else False

            # Assuming response is a list of tuples
            # You may need to unpack the tuples
            content = chunk.choices[0].delta.content

            response_data = {
                "model": "glm",
                "created_at": datetime.datetime.now().isoformat(),
                "done": end,
                "message": {
                    "role": "assistant",
                    "content": content if content is not None else "",
                    "images": None,
                }
                if content is not None
                else None,
            }

            # If end is True, build final_response
            if end:
                # Build final_response
                response_data.update(
                    {
                        "total_duration": 4883583458,
                        "load_duration": 1334875,
                        "prompt_eval_count": 26,
                        "prompt_eval_duration": 342546000,
                        "eval_count": 282,
                        "eval_duration": 4535599000,
                    }
                )

            json_response_data = json.dumps(response_data)


            yield f"{json_response_data}\n"



from datetime import datetime
import random
import string


def parse_model_name(section_name):
    parts = section_name.split("-", 1)
    return f"{parts[0]}:{parts[1]}" if len(parts) > 1 else section_name


def generate_random_digest(length=64):
    return "".join(random.choices(string.hexdigits.lower(), k=length))


def list_models(model_name):
    try:
        model_data = {
            "name": parse_model_name(model_name),
            "modified_at": datetime.now().isoformat(),
            "size": 1000000000,  
            "digest": generate_random_digest(),
            "details": {
                "format": "gguf",  
                "family": "llama",  
                "families": None,
                "parameter_size": "14b",  
                "quantization_level": "Q4_0",  
            },
        }
        return {"models": [model_data]}
    except Exception as e:
        raise Exception(str(e))



from abc import ABC, abstractmethod
import aiohttp
from typing import List, Dict, Any, AsyncGenerator
from ..define import ChatRequest


class BaseModelService(ABC):
    def __init__(self, provider: str, url: str, api_key: str):
        self.provider = provider
        self.url = url
        self.api_key = api_key

    @abstractmethod
    async def chat(self, chat_request: ChatRequest) -> AsyncGenerator[str, None]:
        """
        Abstract method for implementing streaming chat functionality.

        Parameters:
        - messages: List of chat messages
        - model_name: Model name
        - kwargs: Other optional parameters

        Returns:
        - Asynchronous generator producing strings formatted as SSE
        """
        pass

    async def list_models(self) -> List[Dict[str, Any]]:
        """
        Get the list of available models.

        Returns:
        - List of dictionaries containing model information
        """
        async with aiohttp.ClientSession() as session:
            async with session.get(f"{self.url}/api/tags") as response:
                if response.status != 200:
                    raise Exception(f"API 请求失败: {response.status}")
                data = await response.json()
                return data.get("models", [])

    def get_model_info(self) -> Dict[str, str]:
        """
        获取模型信息。

        返回:
        - 包含模型信息的字典
        """
        return {"provider": self.provider, "url": self.url}

    def __str__(self) -> str:
        return f"{self.provider} {self.url}"



from .base import BaseModelService
import aiohttp
import json
from typing import List, Dict, Any, AsyncGenerator


class DeepseekModelService(BaseModelService):
    async def chat(
        self, messages: List[Dict[str, Any]], **kwargs
    ) -> AsyncGenerator[str, None]:
        """
        Implement streaming chat functionality for Deepseek models.

        Parameters:
        - messages: List of chat messages
        - kwargs: Other optional parameters

        Returns:
        - Asynchronous generator producing strings formatted as SSE
        """
        headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.api_key}",
        }

        data = {
            "model": kwargs.get("model", "deepseek-chat"),
            "messages": messages,
            "temperature": kwargs.get("temperature", 0.7),
            "top_p": kwargs.get("top_p", 1),
            "stream": True,
            "max_tokens": kwargs.get("max_tokens", 2048),
        }

        async with aiohttp.ClientSession() as session:
            async with session.post(
                f"{self.url}/chat/completions", headers=headers, json=data
            ) as response:
                if response.status != 200:
                    raise Exception(f"API request failed: {response.status}")

                async for line in response.content:
                    decoded_line = line.decode("utf-8").strip()
                    if decoded_line.startswith("data: "):
                        json_data = json.loads(decoded_line[6:])
                        if json_data["choices"][0]["finish_reason"] is not None:
                            break
                        content = json_data["choices"][0]["delta"].get("content", "")
                        if content:
                            yield f"data: {json.dumps({'content': content})}\n\n"



import toml
from .services import create_model_service


def init_model_service(config_file: str, model_name: str):
    try:
        # 读取配置文件
        models_list = toml.load(open(config_file, "r"))

        if model_name not in models_list:
            raise ValueError(f"Model {model_name} not found in the configuration file")

        model_config = models_list[model_name]
        provider = model_config.get("provider")
        service_url = model_config.get("url")
        api_key = model_config.get("api_key")

        model_service = create_model_service(provider, service_url, api_key)

        return model_service
    except Exception as e:
        raise ValueError(f"Error loading configuration or creating model service: {str(e)}")



import os
import click
import uvicorn
from contextlib import asynccontextmanager
from fastapi import FastAPI, HTTPException, Request
from fastapi.responses import StreamingResponse, JSONResponse
from .define import ChatRequest
from .config import init_model_service
from .models import list_models

# Use environment variables or default values to set the configuration file path
DEFAULT_CONFIG_PATH = os.environ.get("OLLAMA_PROXY_CONFIG", "keys.toml")


@asynccontextmanager
async def lifespan(app: FastAPI):
    print("Server is starting")
    app.state.config_path = os.environ.get("CONFIG_PATH", DEFAULT_CONFIG_PATH)
    app.state.model_name = os.environ.get("MODEL_NAME", "default_model")

    yield
    print("Server is shutting down")
    # Here you can clean up resources, such as closing database connections


app = FastAPI(lifespan=lifespan)


@app.post("/api/chat")
async def chat(chat_request: ChatRequest, request: Request):
    if not hasattr(request.app.state, "model_name") and hasattr(
        request.app.state, "config_path"
    ):
        print("app.state is missing model_name or config_path attribute")
        return JSONResponse(
            content={"error": "app.state is missing model_name or config_path attribute"}
        )

    model_name = request.app.state.model_name
    config_path = request.app.state.config_path

    model_service = init_model_service(config_path, model_name)

    try:
        stream_generator = model_service.chat(chat_request)
        if chat_request.stream:
            return StreamingResponse(stream_generator, media_type="text/event-stream")
        else:
            response_content = ""
            async for chunk in stream_generator:
                response_content += chunk
            return JSONResponse(content={"response": response_content})
    except Exception as e:
        raise HTTPException(status_code=500, detail=f"Error processing chat request: {str(e)}")


@app.get("/api/tags")
async def get_models(request: Request):
    try:
        model_name = request.app.state.model_name
        result = list_models(model_name)
        return JSONResponse(content=result)
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


@app.get("/api/ping")
async def ping():
    """
    A simple ping-pong test endpoint
    """
    return JSONResponse(content={"message": "pong"})


@click.command()
@click.argument("model_name")
@click.argument("config", default=DEFAULT_CONFIG_PATH, help="Path to the Toml configuration file")
@click.option("--host", default="127.0.0.1", help="Server host address")
@click.option("--port", default=8000, type=int, help="Server port")
@click.option("--reload", is_flag=True, help="Enable hot reloading")
def run(model_name, config, host, port, reload):
    """Run a specific model"""

    # Set environment variables for use in the startup event
    os.environ["CONFIG_PATH"] = config
    os.environ["MODEL_NAME"] = model_name

    uvicorn.run(
        "ollama_proxy.main:app",
        host=host,
        port=port,
        reload=reload,
    )


if __name__ == "__main__":
    run()



from pydantic import BaseModel, Field
from typing import List, Optional, Dict, Union, Any


class Image(BaseModel):
    data: str


class ToolCall(BaseModel):
    id: str
    type: str
    function: Optional[Dict[str, Any]] = None
    retrieval: Optional[Dict[str, Any]] = None
    web_search: Optional[Dict[str, Any]] = None


class Message(BaseModel):
    role: str
    content: str
    images: Optional[List[Image]] = None
    tool_calls: Optional[List[ToolCall]] = None


class Image(BaseModel):
    data: str


class ToolCall(BaseModel):
    id: str
    type: str
    function: Optional[Dict[str, Any]] = None
    retrieval: Optional[Dict[str, Any]] = None
    web_search: Optional[Dict[str, Any]] = None


class Choice(BaseModel):
    index: int
    finish_reason: str
    delta: Dict[str, Union[str, None]]
    tool_calls: Optional[List[ToolCall]] = None


class Usage(BaseModel):
    prompt_tokens: int
    completion_tokens: int
    total_tokens: int


class WebSearchResult(BaseModel):
    icon: str
    title: str
    link: str
    media: str
    content: str


class ChatRequest(BaseModel):
    model: str
    messages: List[Message]
    request_id: Optional[str] = None
    do_sample: Optional[bool] = True
    stream: Optional[bool] = False
    temperature: Optional[float] = 0.95
    top_p: Optional[float] = 0.7
    max_tokens: Optional[int] = 1024
    stop: Optional[List[str]] = Field(default_factory=list)
    tools: Optional[List[ToolCall]] = Field(default_factory=list)
    tool_choice: Optional[Union[str, Dict[str, Any]]] = "auto"
    user_id: Optional[str] = None
    keep_alive: Optional[Union[str, int]] = "5m"


class ChatResponse(BaseModel):
    id: str
    created: int
    model: str
    choices: List[Choice]
    usage: Optional[Usage] = None
    web_search: Optional[List[WebSearchResult]] = None
    done: bool
    total_duration: Optional[int] = None
    load_duration: Optional[int] = None
    prompt_eval_count: Optional[int] = None
    prompt_eval_duration: Optional[int] = None
    eval_count: Optional[int] = None
    eval_duration: Optional[int] = None
